#include "udx_api.hpp"                               	// For extensions API
using namespace std;                                 	// Everything in std
using namespace udx2;                                	// Everything in udx

// create a concat function that concatenates two string values 

struct concat : Function
{
  void apply(const Row& a,Row& r)
  {
    r.setString(0, string(a.getString(0)) + string(a.getString(1)));
  }
};
extern "C" ExtensionFactory* udx_concat() { return new FactoryFor<concat>(); }

// create a usd_to_euro function that converts USD to EUR

struct usd_to_euro : Function
{
  void apply(const Row& a,Row& r)
  {
    auto exchange_rate = 0.81;
    auto [udt,tag] 	= a.getUDT(0);
 
    if (tag=="$" && !udt.empty())
	{
      istringstream i{string(udt)};
      ostringstream o;o.precision(2);o.setf(ios::fixed);
      double    	d;
 
      i >> d;
      o << d * exchange_rate ;
 
      r.setUDT(0,{o.str(),"€"});
	}
  }
};
extern "C" ExtensionFactory* udx_usd_to_euro() { return new FactoryFor<usd_to_euro>(); }

// create a sort_string function that sorts the words into a string

struct sortstr : Function
{
  void apply(const Row& a,Row& r)
  {
    ostringstream             o;
    istringstream             i(string(a.getString(0)));
    istream_iterator<string>  b(i);
    vector<string>            tokens(b,istream_iterator<string>());
 
    sort(begin(tokens),end(tokens));
 
    copy(begin(tokens),end(tokens),ostream_iterator<string>(o," "));
 
    r.setString(0,o.str().c_str());
  }
};
extern "C" ExtensionFactory* udx_sortstr() { return new FactoryFor<sortstr>(); }

